import json
import pickle
import gzip
from base64 import b64encode, b64decode


def compress(original_data):
    pickled_data = pickle.dumps(original_data)
    gzipped_data = gzip.compress(pickled_data,compresslevel=1)
    b64_data = b64encode(gzipped_data)
    return b64_data.decode()


def decompress(compressed_data):
    b64_data = bytes(compressed_data, "utf-8")
    gzipped_data = b64decode(b64_data)
    pickled_data = gzip.decompress(gzipped_data)
    original_data = pickle.loads(pickled_data)
    return original_data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import time
import traceback
from .tournament_client import TournamentConnection
from colorama import Fore, Style
from halo import Halo


class ColosseumTournamentAgent:
    def __init__(self, policy, latency=0,  games=["__all__"], port=12345,
                 direct=False, host="localhost", maximum_rounds=10000, debug=False):
        self.host = host
        self.policy = policy
        self.port = port
        self.botname = ''
        self.key = ''
        self.latency = latency
        self.direct = direct
        self.games = games
        self.tournament = None
        self.maximum_rounds = maximum_rounds
        self.debug = debug

    def connect(self, botname, key):
        self.botname = botname
        self.key = key
        try:
            # TODO: Use policy for test
            # Test that policy runs without errors in local environments
            self.tournament = TournamentConnection(
                self.host, self.port, self.botname, self.key,
                available_games=self.games, debug=self.debug
            )
            print(Fore.GREEN + "Bot: {}'s policy has passed environment verifications".format(self.botname))
            print(Style.RESET_ALL)
        except Exception:
            print(Fore.RED + "Bot: {}'s policy has failed verification testing in environment: ".format(self.botname))
            print(Style.RESET_ALL)
            quit()

    def run(self):
        # Connect to tournament server each round, until the end signal is received.
        try:
            env = self.tournament.next_match()
            if self.debug:
                print(env)
        except Exception as e:
            print(Fore.RED + str(e))
            print(traceback.format_exc())
            print(Style.RESET_ALL)
            quit()
        current_round = 1
        while env is not None:
            surprises = []
            term = False
            trunc = False
            timestep = 0
            rew = info = None
            obs = env.reset()
            while not (term or trunc):
                if timestep % 100 == 0 and self.debug:
                    print(f"{self.botname}: Timestep {timestep}\n")
                time.sleep(self.latency / 1000)  # Used to simulate network latency
                (action, surprise) = self.policy(obs, rew, term, trunc, info)  # receive action and surprise from user
                obs, rew, term, trunc, info = env.step(action)  # Send action to game server
                surprises.append(surprise)  # Collect surprise for later use when game server supports
                timestep += 1
            print(Fore.GREEN + f"Round {current_round} complete")
            print(Style.RESET_ALL)
            current_round += 1
            if current_round > self.maximum_rounds:
                env = None
            else:
                env = self.tournament.next_match()



from pettingzoo.atari import (boxing_v2,
                              combat_tank_v2,
                              combat_plane_v2,
                              double_dunk_v3,
                              entombed_competitive_v3,
                              entombed_cooperative_v3,
                              flag_capture_v2,
                              joust_v3,
                              ice_hockey_v2,
                              mario_bros_v3,
                              othello_v3,
                              basketball_pong_v3,
                              pong_v3,
                              foozpong_v3,
                              quadrapong_v4,
                              volleyball_pong_v3,
                              space_invaders_v2,
                              space_war_v2,
                              surround_v2,
                              tennis_v3,
                              video_checkers_v4,
                              wizard_of_wor_v3,
                              warlords_v3)
from pettingzoo.classic import (chess_v5,
                                connect_four_v3,
                                gin_rummy_v4,
                                go_v5,
                                hanabi_v4,
                                leduc_holdem_v4,
                                rps_v2,
                                texas_holdem_v4,
                                texas_holdem_no_limit_v6,
                                tictactoe_v3)

from supersuit import frame_skip_v0, frame_stack_v1
from pettingzoo.utils import aec_to_parallel, turn_based_aec_to_parallel

all_environments = {
    "boxing_v2": boxing_v2,
    "combat_tank_v2": combat_tank_v2,
    "combat_plane_v2": combat_plane_v2,
    "double_dunk_v3": double_dunk_v3,
    "entombed_cooperative_v3": entombed_cooperative_v3,
    "entombed_competitive_v3": entombed_competitive_v3,
    "flag_capture_v2": flag_capture_v2,
    "joust_v3": joust_v3,
    "ice_hockey_v2": ice_hockey_v2,
    "mario_bros_v3": mario_bros_v3,
    "othello_v3": othello_v3,
    "pong_v3": pong_v3,
    "basketball_pong_v3": basketball_pong_v3,
    "foozpong_v3": foozpong_v3,
    "quadrapong_v4": quadrapong_v4,
    "volleyball_pong_v3": volleyball_pong_v3,
    "space_invaders_v2": space_invaders_v2,
    "space_war_v2": space_war_v2,
    "surround_v2": surround_v2,
    "tennis_v3": tennis_v3,
    "video_checkers_v4": video_checkers_v4,
    "wizard_of_wor_v3": wizard_of_wor_v3,
    "warlords_v3": warlords_v3,
    "connect_four_v3": connect_four_v3,
    "gin_rummy_v4": gin_rummy_v4,
    "go_v5": go_v5,
    "hanabi_v4": hanabi_v4,
    "leduc_holdem_v4": leduc_holdem_v4,
    #"rps_v2": rps_v2,
    "texas_holdem_v4": texas_holdem_v4,
    "texas_holdem_no_limit_v6": texas_holdem_no_limit_v6,
    "tictactoe_v3": tictactoe_v3,
}


def get_num_agents(name, env):
    count_env = env.env()
    count_env.reset()
    return count_env.num_agents


env_num_players = {name: get_num_agents(name, env) for name, env in all_environments.items()}

MAX_CYCLES = 10000


def make_test_env(game_id, seed=None, render_mode=None, turn_based=False):
    env = all_environments[game_id]
    # Check if game can be played with parallel API
    test_env = env.env()
    turn_based = not test_env.metadata["is_parallelizable"]
    if turn_based:
        env = env.env(render_mode=render_mode)
        env = turn_based_aec_to_parallel(env)
    else:
        # TODO: Redo preprocessing by environment class
        env = env.env(render_mode=render_mode)
        env = frame_stack_v1(env, 4)
        env = frame_skip_v0(env, 4)
        env = aec_to_parallel(env)
    return env, turn_based



from setuptools import find_packages, setup
import codecs
import os

VERSION = '0.0.3'
DESCRIPTION = 'Umshini Client API'

setup(
    name='umshini',
    version=VERSION,
    author='umshini team',
    author_email="j.k.terry@swarmlabs.com",
    description=DESCRIPTION,
    url='https://github.com/umshini/umshini',
    #long_description=long_description,
    #long_description_content_type="text/markdown",
    #keywords=["Reinforcement Learning", "game", "RL", "AI", "gym"],
    #python_requires=">=3.6, <3.10",
    #data_files=[("", ["LICENSE.txt"])],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'colorama',
        'pettingzoo==1.22.2',
        'supersuit==3.7.0',
        'gymnasium',
        'numpy',
        'halo',
    ],
    extras_require={
        "atari": ['pettingzoo[atari]==1.22.2'],
        "classic": ['pettingzoo[classic]==1.22.2'],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ]
)



from .example_client import ColosseumTournamentAgent
from .tournament_client import TestEnv
from colorama import Fore, Style


def create_and_run(botname, user_key):
    agent = ColosseumTournamentAgent(maximum_rounds=100)
    agent.connect(botname, user_key)
    agent.run()


def connect(environment, botname, user_key, user_policy, debug=False):
    """
    User end function to add their RL policy

    Passed function accepts parameters:
        policy(observation, reward, done, info)

    Passed function returns action
    """

    # agent = ColosseumTournamentAgent(policy=user_policy, games=[environment],
    #                                  host="34.70.234.149", port=8803, maximum_rounds=100)
    agent = ColosseumTournamentAgent(policy=user_policy, games=[environment],
                                     maximum_rounds=100, debug=debug)
    agent.connect(botname, user_key)
    agent.run()


def test(environment, user_policy):
    test_env = TestEnv(environment)
    term = trunc = False
    obs = rew = info = None
    for _ in range(100):
        try:
            (action, surprise) = user_policy(obs, rew, term, trunc, info)
            obs, rew, term, trunc, info = test_env.step(action)

            if term or trunc:
                print(Fore.GREEN + "Policy has passed verification testing in {}".format(environment))
                print(Style.RESET_ALL)
                break
        except:
            print(Fore.RED + "Policy has failed verification testing")
            print(Style.RESET_ALL)
            quit()
